package com.qmth.themis.business.service.impl;

import java.util.Date;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.bean.exam.LivenessVerifyBean;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.LivenessVerifyCacheBean;
import com.qmth.themis.business.dao.TOeLivenessVerifyHistoryMapper;
import com.qmth.themis.business.entity.TOeLivenessVerifyHistory;
import com.qmth.themis.business.enums.LivenessTypeEnum;
import com.qmth.themis.business.service.TOeLivenessVerifyHistoryService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;

/**
 * @Description: 活体验证记录 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TOeLivenessVerifyHistoryServiceImpl extends ServiceImpl<TOeLivenessVerifyHistoryMapper, TOeLivenessVerifyHistory> implements TOeLivenessVerifyHistoryService {

	@Resource
	RedisUtil redisUtil;
	
	@Override
	public LivenessVerifyBean verify(Long recordId, String type, String actions, Integer retry, Long startTime,
			Long finishTime, Boolean pass) {
		LivenessVerifyCacheBean cache=new LivenessVerifyCacheBean();
		cache.setId(Constants.idGen.next());
		cache.setExamRecordId(recordId);
		cache.setType(LivenessTypeEnum.valueOf(type));
		cache.setActions(actions);
		cache.setRetry(retry);
		cache.setStartTime(new Date(startTime));
		cache.setFinishTime(new Date(finishTime));
		cache.setPass((pass?1:0));
		redisUtil.set(RedisKeyHelper.livenessVerifyCacheKey(cache.getId()), cache);
		LivenessVerifyBean ret=new LivenessVerifyBean();
		ret.setId(cache.getId());
		ret.setUpdateTime(new Date().getTime());
		return ret;
	}
	
	
	@Transactional
	@Override
	public void save(Long id,Long recordId, String type, String actions, Integer retry, Long startTime,
			Long finishTime, Boolean pass) {
		TOeLivenessVerifyHistory entity=new TOeLivenessVerifyHistory();
		entity.setId(id);
		entity.setExamRecordId(recordId);
		entity.setType(LivenessTypeEnum.valueOf(type));
		entity.setActions(actions);
		entity.setRetry(retry);
		entity.setStartTime(new Date(startTime));
		entity.setFinishTime(new Date(finishTime));
		saveOrUpdate(entity);
	}

}
